package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEntities();
            executeQuery();
            executeQuery2();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEntities() {
        System.out.println("-- persisting entities --");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        getEmployees().forEach(em::persist);
        getJobInfoList().forEach(em::persist);
        em.getTransaction().commit();
        em.close();
    }

    private static void executeQuery() {
        System.out.println("-- executing query EXISTS --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e"
                + " WHERE EXISTS (SELECT j from JobInfo j WHERE j.jobName = e.job)");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void executeQuery2() {
        System.out.println("-- executing query NOT EXISTS --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e"
                + " WHERE NOT EXISTS (SELECT j from JobInfo j WHERE j.jobName = e.job)");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static List<JobInfo> getJobInfoList() {
        List<JobInfo> list = new ArrayList<>();
        list.add(JobInfo.create("Developer", "Consultant"));
        list.add(JobInfo.create("Manager", "FullTime"));
        list.add(JobInfo.create("Architect", "Consultant"));
        return list;
    }

    private static List<Employee> getEmployees() {
        List<Employee> employees = new ArrayList<>();
        employees.add(Employee.create("Diana", "Developer"));
        employees.add(Employee.create("Mike", "Manager"));
        employees.add(Employee.create("Tim", "Salesman"));
        employees.add(Employee.create("Jack", "Architect"));
        return employees;
    }
}